import { ModalFormData, ActionFormData } from "@minecraft/server-ui";
import { world, system, ItemTypes } from "@minecraft/server";
import { showForm } from "../functions/showForm"

function showEditForm(player, selectedType, rewardsData, rewards, index, errorMsg = "", lastInput = {}) {
  const selected = rewards[index];

  const form = new ModalFormData().title(`Edit ${selected.item}`);
  form.label(errorMsg ? `§c${errorMsg}` : " ");

  form
    .textField("Item Name", "e.g., Mace", {defaultValue: lastInput.item ?? selected.item})
    .textField("Amount", "e.g., 1", {defaultValue: lastInput.amount ?? selected.amount.toString()})
    .textField("Rarity (0.1–2.99)", "e.g., 0.2", {defaultValue: lastInput.rarity ?? selected.rarity.toString() })
    .textField("Type ID", "e.g., minecraft:mace", {defaultValue: lastInput.typeId ?? selected.typeId })
    .textField("Texture (Optional)", "e.g., textures/items/mace", {defaultValue: lastInput.texture ?? selected.texture ?? ""});

  form.show(player).then(editResult => {
    if (editResult.canceled) return;

    const [_, item, amountRaw, rarityRaw, typeId, texture] = editResult.formValues;

    const amount = parseInt(amountRaw);
    const rarity = parseFloat(rarityRaw);

    const input = { item, amount: amountRaw, rarity: rarityRaw, typeId, texture };

    if (!item || item.length < 1) {
      return showEditForm(player, selectedType, rewardsData, rewards, index, "Item name is required.", input);
    }

    if (isNaN(amount) || amount < 1) {
      return showEditForm(player, selectedType, rewardsData, rewards, index, "Invalid amount. Must be 1 or more.", input);
    }

    if (isNaN(rarity) || rarity < 0.1 || rarity > 2.99) {
      return showEditForm(player, selectedType, rewardsData, rewards, index, "Rarity must be between 0.1 and 2.99.", input);
    }

    if (!/^([a-z0-9_.-]+:)?[a-z0-9_./-]+$/.test(typeId)) {
      return showEditForm(player, selectedType, rewardsData, rewards, index, "Invalid Type ID format.", input);
    }

    try {
      const exists = ItemTypes.get(typeId);
      if (!exists) throw new Error();
    } catch {
      return showEditForm(player, selectedType, rewardsData, rewards, index, "Type ID does not exist in-game.", input);
    }

    rewards[index] = {
      item,
      amount,
      rarity,
      typeId,
      ...(texture ? { texture } : {})
    };

    rewardsData[selectedType] = rewards;
    world.setDynamicProperty("stk.rewards", JSON.stringify(rewardsData));
    player.sendMessage(`§aUpdated reward for ${selectedType} chest: ${item}`);
  });
}

export default {
  name: "config",
  description: "Configure streak settings for this world.",
  usage: "<settings|rewards>",
  op: true,
  run: async (chat, player, args) => {
    if(!args.length || !["settings", "rewards"].includes(args[0].toLowerCase())) return player.sendMessage("§eUsage: s.config <settings|rewards>")
  
  if(args[0].toLowerCase() === "settings") {
    const configRaw = world.getDynamicProperty("stk.config") ?? "{}";
    const config = JSON.parse(configRaw);

    const quest = config.quest ?? true;
    const reward = config.reward ?? true;
    const weekly = config.weekly ?? true;
   const chatName = config.chatName ?? true;
   const nameTag = config.nameTag ?? true;
 
    const form = new ModalFormData()
      .title("Streak Configuration")
      .header("System")
      .toggle("Quest System", {defaultValue: quest})
      .toggle("Reward System", {defaultValue: reward})
      .toggle("Weekly System", {defaultValue: weekly})
      .divider()
      .header("Display")
      .toggle("Display streak on chat names", {defaultValue: chatName})
      .toggle("Display streak on name tags", {defaultValue: nameTag})
      .submitButton("Save")

   player.sendMessage("§ePlease close chat screen to view config form.")
   system.run(async() => {
   await showForm(player, form).then(response => {
      if (response.canceled) return;

      const [h1, newQuest, newReward, newWeekly, div, h2, cn, nt] = response.formValues;
      const newConfig = {
        quest: newQuest,
        reward: newReward,
        weekly: newWeekly,
        chatName: cn,
        nameTag: nt
      };

      world.setDynamicProperty("stk.config", JSON.stringify(newConfig));

      player.sendMessage("§aStreak configuration updated successfully.");
    });
   })
  
  }
if(args[0].toLowerCase() === "rewards") {
    const mainForm = new ActionFormData()
      .title("Chest Rewards Config")
      .body("Choose what system you want to configure:")
      .button("Quest Chest", "textures/ext/chests/quest")
      .button("Weekly Chest", "textures/ext/chests/weekly")
      .button("Reward Chest", "textures/ext/chests/reward");

    player.sendMessage("§ePlease close chat screen to configure chest rewards.");

   system.run(() => {
    showForm(player, mainForm).then(res => {
      if (res.canceled) return;

      const chestTypes = ["quest", "weekly", "reward"];
      const selectedType = chestTypes[res.selection];

      const rewardsData = JSON.parse(world.getDynamicProperty("stk.rewards"));
      const rewards = rewardsData[selectedType];

      const listForm = new ActionFormData()
        .title(`Edit ${selectedType.charAt(0).toUpperCase() + selectedType.slice(1)} Rewards`)
        .body("Select a reward to edit:");

      for (const rw of rewards) {
        listForm.button(`${rw.item} ×${rw.amount} [${rw.rarity}]`);
      }

      listForm.show(player).then(result => {
        if (result.canceled) return;

        showEditForm(player, selectedType, rewardsData, rewards, result.selection);
      });
    });
   })
}
  }
};


